//
//  GameListController.m
//  Passer Rating
//
//  Created by Xcode User on 6/25/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "GameListController.h"
#import "Passer.h"
#import "Game.h"

@implementation GameListController

@synthesize passer;

@synthesize fullNameLabel;
@synthesize passerRatingLabel;
@synthesize currentTeamLabel;
@synthesize datesLabel;
@synthesize attemptsLabel;
@synthesize completionsLabel;
@synthesize yardsLabel;
@synthesize touchdownsLabel;
@synthesize interceptionsLabel;
@synthesize tableView;

static NSNumberFormatter *  sNumberFormat = nil;
static NSDateFormatter *    sDateFormat = nil;

+ (void) initialize
{
    //  Make a formatter for the passer rating
    static dispatch_once_t      formatOnceToken;
    dispatch_once(&formatOnceToken, ^{
        sNumberFormat = [[NSNumberFormatter alloc] init];
        sNumberFormat.numberStyle = NSNumberFormatterDecimalStyle;
        sNumberFormat.minimumFractionDigits = 1;
        sNumberFormat.maximumFractionDigits = 1;
        
        sDateFormat = [[NSDateFormatter alloc] init];
        sDateFormat.dateStyle = NSDateFormatterShortStyle;
    });
}

+ (NSArray *) integerProperties
{
    static NSArray *        sNumericProperties = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sNumericProperties = [[NSArray alloc] initWithObjects:
                              @"attempts", @"completions",
                              @"yards", @"touchdowns", 
                              @"interceptions", nil];
    });
    
    return sNumericProperties;
}

+ (NSArray *) stringProperties
{
    static NSArray *        sStringProperties = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sStringProperties = [[NSArray alloc] initWithObjects:
                              @"fullName", @"currentTeam", nil];
    });
    
    return sStringProperties;
}

- (id) initWithNibName: (NSString *) nibNameOrNil
                bundle: (NSBundle *) nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void) dealloc 
{
    [arrangedGamesCache release];
    [passer release];
    [fullNameLabel release];
    [passerRatingLabel release];
    [currentTeamLabel release];
    [datesLabel release];
    [attemptsLabel release];
    [completionsLabel release];
    [yardsLabel release];
    [touchdownsLabel release];
    [interceptionsLabel release];
    [tableView release];
    [super dealloc];
}

- (void) loadProperties
{
    NSString *      fieldName;
    NSString *      strValue;
    NSNumber *      numValue;
    
    //  Set all the string labels
    for (NSString * prop
         in [GameListController stringProperties]) {
        NSString    *value = [self.passer valueForKey: prop];
        fieldName = [prop stringByAppendingString: @"Label"];
        [[self valueForKey: fieldName] setText: value];
    }
    
    //  Set all the integer labels
    for (NSString * prop
         in [GameListController integerProperties]) {
        numValue = [self.passer valueForKey: prop];
        
        fieldName = [prop stringByAppendingString: @"Label"];
        [[self valueForKey: fieldName] setText: 
            [NSString stringWithFormat: @"%d", numValue.intValue]];
    }
    
    
    //  Set the passer-rating label
    strValue = [sNumberFormat stringFromNumber:
                                self.passer.passerRating];
    self.passerRatingLabel.text = strValue;
        
    //  Set the date-range label
    strValue = [NSString stringWithFormat: @"%@ - %@",
                [sDateFormat stringFromDate: self.passer.firstPlayed],
                [sDateFormat stringFromDate: self.passer.lastPlayed]];
    self.datesLabel.text = strValue;
}

- (NSArray *) arrangedGames
{
    static NSArray *    sByDate = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSSortDescriptor *  desc = 
            [NSSortDescriptor sortDescriptorWithKey: @"whenPlayed"
                                          ascending: YES];
        sByDate = [[NSArray alloc] initWithObjects: desc, nil];
    });
    
    if (! arrangedGamesCache) {
        arrangedGamesCache = [self.passer.games 
                              sortedArrayUsingDescriptors: sByDate];
        [arrangedGamesCache retain];
    }
    return arrangedGamesCache;
}

- (void) setPasser: (Passer *) newPasser
{
    if (passer != newPasser) {
        [passer release];
        passer = [newPasser retain];
        
        [arrangedGamesCache release];
        arrangedGamesCache = nil;
        
        [self loadProperties];
        [self.tableView reloadData];
    }
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self loadProperties];
}

- (void)viewDidUnload
{
    [self setFullNameLabel:nil];
    [self setPasserRatingLabel:nil];
    [self setCurrentTeamLabel:nil];
    [self setDatesLabel:nil];
    [self setAttemptsLabel:nil];
    [self setCompletionsLabel:nil];
    [self setYardsLabel:nil];
    [self setTouchdownsLabel:nil];
    [self setInterceptionsLabel:nil];
    [self setTableView:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - UITableViewDataSource

- (NSInteger) tableView: (UITableView *) tableView
  numberOfRowsInSection: (NSInteger) section
{
    return self.passer.games.count;
}

//  Tag values for the labels in the game cell
#define CELL_RATING_LABEL   1
#define CELL_SCORE_LABEL    2
#define CELL_STATS_LABEL    3

- (UITableViewCell *) tableView: (UITableView *) aTableView
          cellForRowAtIndexPath: (NSIndexPath *) indexPath 
{
    static NSString *cellIdentifier = @"Game Cell";

    UITableViewCell *   cell = 
                [tableView dequeueReusableCellWithIdentifier: cellIdentifier];
    
    //  Could we get the cell out of recycling?
    if (cell == nil) {
        //  No. Load a fresh cell from the NIB.
        NSArray *   nibContents;
        nibContents = [[UINib nibWithNibName: @"GameTableCell" bundle: nil]
                       instantiateWithOwner: nil options: nil];
        cell = [nibContents objectAtIndex: 0];
    }
    
    //  Pull data from this row's Game.
    Game *      game = [self.arrangedGames objectAtIndex: indexPath.row];
    UILabel *   label;
    NSString *  content;
    
    //  Find the labels by their tags.
    //  For each, format the data and set the label text.
    label = (UILabel *) [cell viewWithTag: CELL_RATING_LABEL];
    label.text = [sNumberFormat stringFromNumber: game.passerRating];
    
    label = (UILabel *) [cell viewWithTag: CELL_SCORE_LABEL];
    content = [NSString stringWithFormat: 
               @"%@ %d\n%@ %d\n%@",
               game.ourTeam, game.ourScore.intValue,
               game.theirTeam, game.theirScore.intValue,
               [sDateFormat stringFromDate: game.whenPlayed]];
    label.text = content;
    
    label = (UILabel *) [cell viewWithTag: CELL_STATS_LABEL];
    content = [NSString stringWithFormat:
               @"%d/%d - %d yd - %d TD - %d INT",
               game.completions.intValue, game.attempts.intValue,
               game.yards.intValue, game.touchdowns.intValue,
               game.interceptions.intValue];
    label.text = content;

    return cell;
}

@end
