//
//  RatingTest.m
//  Passer Rating
//
//  Created by Xcode User on 7/9/11.
//  Copyright 2011 Fritz Anderson. All rights reserved.
//

#import "RatingTest.h"
#import "rating.h"

typedef enum {
    kCompletion,
    kYardage,
    kTouchdown,
    kInterception
} ComponentKey;

@interface RatingTest ()
@property(nonatomic, retain) NSMutableArray *   games;
@end

@implementation RatingTest
@synthesize games;

static
double pinRating(double component)
{
	if (component < 0.0)
		return 0.0;
	else if (component > 2.375)
		return 2.375;
	else
		return component;
}

- (double) calculateFromRating: (ComponentKey) key attempts: (int) atts stat: (int) stat
{
    double      retval = 0.0;
    
    switch (key) {
        case kCompletion:
            retval =  (((double) stat / atts) * 100.0 - 30.0) / 20.0;
            retval = pinRating(retval);
            break;
            
        case kYardage:
            retval = (((double) stat / atts) - 3.0) / 4.0;
            retval = pinRating(retval);
            break;
            
        case kTouchdown:
            retval = 20.0 * (double) stat / atts;
            retval = pinRating(retval);
            break;
            
        case kInterception:
            retval =  2.375 - (25.0 * (double) stat / atts);
            retval = pinRating(retval);
            break;
            
        default:
            break;
    }
    
    return retval;
}

- (void) setUp
{
    NSError *       error;
    self.games = [NSMutableArray array];
    NSString *  dataPath = [[NSBundle bundleForClass: [self class]]
                            pathForResource: @"2010-data-calculated"
                            ofType: @"csv"];
    SimpleCSVFile * csvFile = [[SimpleCSVFile alloc] initWithPath: dataPath];
    csvFile.delegate = self;
    BOOL        success = [csvFile run: &error];
    STAssertTrue(success, @"Parse should succeed");
    [csvFile release];
}

- (BOOL) csvFile: (SimpleCSVFile *) file
	  readValues: (NSDictionary *) values
		   error: (NSError **) error
{
    [self.games addObject: values];
    return YES;
}

- (void) tearDown
{
    self.games = nil;
}

- (void) testCalculation
{
    int         attempts, completions, interceptions, touchdowns, yards;
    double      component, expectedComponent, rating;
    int         index = 0;
    
    for (NSDictionary * game in self.games) {
        double  accum = 0.0;
        
        attempts = [[game objectForKey: @"attempts"] intValue];
        
        completions = [[game objectForKey: @"completions"] intValue];
        component = [self calculateFromRating: kCompletion attempts: attempts stat: completions];
        accum += component;
        expectedComponent = [[game objectForKey: @"cComp"] doubleValue];
        STAssertEqualsWithAccuracy(component, expectedComponent, 0.01, @"Completions for game %d (%d / %d)", index, completions, attempts);
        
        interceptions = [[game objectForKey: @"interceptions"] intValue];
        component = [self calculateFromRating: kInterception attempts: attempts stat: interceptions];
        accum += component;
        expectedComponent = [[game objectForKey: @"intComp"] doubleValue];
        STAssertEqualsWithAccuracy(component, expectedComponent, 0.01, @"Interceptions for game %d (%d / %d)", index, interceptions, attempts);

        touchdowns = [[game objectForKey: @"touchdowns"] intValue];
        component = [self calculateFromRating: kTouchdown attempts: attempts stat: touchdowns];
        accum += component;
        expectedComponent = [[game objectForKey: @"tdComp"] doubleValue];
        STAssertEqualsWithAccuracy(component, expectedComponent, 0.01, @"Touchdowns for game %d (%d / %d)", index, touchdowns, attempts);
        
        yards = [[game objectForKey: @"yards"] intValue];
        component = [self calculateFromRating: kYardage 
                                     attempts: attempts 
                                         stat: yards];
        accum += component;
        expectedComponent = [[game objectForKey: @"yComp"] doubleValue];
        STAssertEqualsWithAccuracy(component, expectedComponent, 0.01, 
                                   @"Yardage for game %d (%d / %d)", 
                                   index, yards, attempts);

        component = (100.0 / 6.0) * accum; 
        expectedComponent = [[game objectForKey: @"rating"] doubleValue];
        STAssertEqualsWithAccuracy(component, expectedComponent, 0.051,
                                   @"Rating for game %d", index);
        
        rating = passer_rating(completions, attempts,
                               yards, touchdowns, interceptions);
        STAssertEqualsWithAccuracy(rating, expectedComponent, 0.051,
                                   @"passer_rating for game %d", index);

        index++;
    }
}
@end
